using MailKit.Net.Pop3;
using Microsoft.AspNetCore.Http;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.Extensions.Logging;
using System;
using System.Linq;

namespace Pop3ToADefHelpDesk
{
    public static class Pop3ADefHelpDeskEmails
    {
        // "0 */5 * * * *" to run once every 5 minutes
        // "0 */1 * * * *" to run once a minute
        [FunctionName("Pop3ADefHelpDeskEmails")]
        public static void Run(
            [TimerTrigger("0 */5 * * * *")]TimerInfo myTimer,
            ILogger log)
        {
            log.LogInformation("function processed a request.");

            string _POP3Server =
                Environment.GetEnvironmentVariable("POP3Server");
            int _POP3Port =
                Convert.ToInt32(Environment.GetEnvironmentVariable("POP3Port"));
            bool _POP3UseSSL =
                Convert.ToBoolean(Environment.GetEnvironmentVariable("POP3UseSSL"));
            string _POP3Username =
                Environment.GetEnvironmentVariable("POP3Username");
            string _POP3Password =
                Environment.GetEnvironmentVariable("POP3Password");
            string _APIWebAddress =
                Environment.GetEnvironmentVariable("APIWebAddress");

            // ********** GET AuthToken from ADefHelpDesk
            // (needed to send the emails to ADefHelpDesk)

            DTOApiToken paramApiToken = new DTOApiToken();
            paramApiToken.userName =
                Environment.GetEnvironmentVariable("APIUserName");
            paramApiToken.password =
                Environment.GetEnvironmentVariable("APIPassword");
            paramApiToken.applicationGUID =
                Environment.GetEnvironmentVariable("ApplicationGUID");

            // Call the GetAuthToken method and retreive the
            // BearerToken (JWT - Auth Token)
            // (required to make the calls to ADefHelpDesk to save the messages)
            string BearerToken =
                GetAuthToken.GetAuthTokenFromADefHelpDesk(
                    _APIWebAddress, paramApiToken).Result;

            using (var client = new Pop3Client())
            {
                // ********** GET EMAILS

                // Configure connection to Email Server
                // For demo-purposes, accept all SSL certificates 
                // (in case the server supports STARTTLS)
                client.ServerCertificateValidationCallback = (s, c, h, e) => true;

                // Email Server settings
                client.Connect(_POP3Server, _POP3Port, _POP3UseSSL);

                // Note: since we don't have an OAuth2 token, disable
                // the XOAUTH2 authentication mechanism.
                client.AuthenticationMechanisms.Remove("XOAUTH2");

                // Pass username and password to email server
                client.Authenticate(_POP3Username, _POP3Password);

                // Loop through each email
                for (int i = 0; i < client.Count; i++)
                {
                    // message contains the entire contents of the email 
                    // including any attachments
                    var message = client.GetMessage(i);

                    log.LogInformation(
                        $"Got Message: {message.Subject} - {message.MessageId}");

                    // ********** Save Email To ADefHelpDesk

                    // We need to create a Task and a TaskDetail
                    DTOTask objDTOTask = new DTOTask();
                    DTOTaskDetail objDTOTaskDetail = new DTOTaskDetail();

                    // Create a Task
                    objDTOTask.description = message.Subject;
                    objDTOTask.priority = "Normal";
                    objDTOTask.requesterEmail =
                        message.From.Mailboxes.FirstOrDefault().Address;
                    objDTOTask.requesterName =
                        message.From.Mailboxes.FirstOrDefault().Name;
                    objDTOTask.createdDate = DateTime.Now.ToShortDateString();
                    objDTOTask.requesterUserId = -1;
                    objDTOTask.status = "New";
                    objDTOTask.sendEmails = true;

                    // Create a Task Detail
                    objDTOTaskDetail.taskDetailDescription
                        = $"From {objDTOTask.requesterName} [{objDTOTask.requesterEmail}] ";

                    // Send the message to ADefHelpDesk
                    var response = CreateTicket.CreateADefHelpDeskTicket(
                        objDTOTask,
                        objDTOTaskDetail,
                        message,
                        _APIWebAddress,
                        BearerToken);

                    // Check the result
                    if (response.Result.Success)
                    {
                        log.LogInformation(
                            $"Message Saved: {message.Subject} - {message.MessageId}");

                        // [OPTIONAL] Mark the message for deletion
                        // client.DeleteMessage(i);
                    }
                    else
                    {
                        // There was an error
                        log.LogInformation(
                            $"Error saving message: {response.Result.StatusMessage}");
                    }
                }

                log.LogInformation("client.Disconnect");
                client.Disconnect(true);
            }
        }
    }
}